package in.slanglabs;

import android.os.Bundle;
import android.util.Log;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.ReadableMapKeySetIterator;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.modules.core.DeviceEventManagerModule;
import in.slanglabs.assistants.flight.AssistantConfiguration;
import in.slanglabs.assistants.flight.AssistantError;
import in.slanglabs.assistants.flight.FlightStatusQuery;
import in.slanglabs.assistants.flight.SlangFlightAssistant;
import in.slanglabs.platform.SlangBuddy;
import in.slanglabs.platform.SlangSession;
import in.slanglabs.platform.action.SlangAction;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

public class RNSlangFlightAssistantModule extends ReactContextBaseJavaModule {

  private static final String TAG = "RNSlangFlightAssistant";
  private static final String CONFIG_BUDDY_ID = "buddyId";
  private static final String CONFIG_API_KEY = "apiKey";
  private static final String CONFIG_LOCALE = "locale";
  private static final String CONFIG_REQUESTED_LOCALES = "requestedLocales";

  private static final String ENTITY_SRC_AIRPORT = "src_airport";
  private static final String ENTITY_DEST_AIRPORT = "dest_airport";
  private static final String ENTITY_STATUS_DATE = "status_date";
  private static final String ENTITY_FLIGHT_NUMBER = "flight_number";

  private ReadableMap mConfigOptions;
  private SlangSession mCurrentSession;
  private final ReactApplicationContext mReactContext;

  public RNSlangFlightAssistantModule(ReactApplicationContext reactContext) {
    super(reactContext);
    this.mReactContext = reactContext;
  }

  @Override
  public String getName() {
    return "RNSlangFlightAssistant";
  }

  /**
   * Initialize Slang
   *
   * @param configOptions Configurations
   */
  @ReactMethod
  public void initialize(ReadableMap configOptions) {

    Log.i(TAG, "initialize");
    this.mConfigOptions = configOptions;

    // Initialize SlangBuddy
    AssistantConfiguration config =
        new AssistantConfiguration.Builder()
            .setBuddyId(getStringConfig(CONFIG_BUDDY_ID))
            .setAPIKey(getStringConfig(CONFIG_API_KEY))
            .setDefaultLocale(RNSlangLocaleMap.getDefaultLocale(
                getStringConfig(CONFIG_LOCALE)))
            .setRequestedLocales(RNSlangLocaleMap.getRequestedLocales(
                getStringListConfig(CONFIG_REQUESTED_LOCALES)))
            .build();

    SlangFlightAssistant.initialize(getCurrentActivity().getApplication(),
                                    config);

    Log.i(TAG, "Registering Action;");
    SlangFlightAssistant.setAction(new SlangFlightAssistant.Action() {
      @Override
      public Status onTrackStatus(FlightStatusQuery flightStatusQuery, SlangSession slangSession, Bundle extras) {
        mCurrentSession = slangSession;
        mCurrentSession.waitForActionCompletion();
        WritableMap map = mapFlightStatusQuery(flightStatusQuery);
        Log.i(TAG, "onTrackStatus");
        sendEvent("onTrackStatus", map);
        return Status.SUCCESS;
      }

      @Override
      public void onAssistantError(AssistantError assistantError) {
        Log.i(TAG, "onAssistantError");
        WritableMap params = Arguments.createMap();
        params.putInt("type", assistantError.getType());
        params.putString("description", assistantError.getDescription());
        sendEvent("onAssistantError", params);
      }
    });
  }

  /**
   * Notify Slang of a action's success/failure status for Slang to show
   * completion statement
   *
   * @param isSuccess To indicate success/failure.
   */
  @ReactMethod
  public void notifyActionCompleted(boolean isSuccess) {
    if (mCurrentSession != null) {
      try {
        mCurrentSession.notifyActionCompleted(isSuccess ? SlangAction.Status.SUCCESS : SlangAction.Status.FAILURE);
      } catch (SlangBuddy.NotifyBeforeWaitException e) {
        Log.e(TAG, "notifyActionCompleted", e);
      }
    }
  }

  @ReactMethod
  public void show() {
    SlangFlightAssistant.getUI().show(getCurrentActivity());
  }

  @ReactMethod
  public void hide() {
    SlangFlightAssistant.getUI().hide(getCurrentActivity());
  }

  @ReactMethod
  public void notifyNonVoiceStatusQuery(ReadableMap statusQuery) {
    String srcAirport = statusQuery.getString(ENTITY_SRC_AIRPORT);
    String destAirport = statusQuery.getString(ENTITY_DEST_AIRPORT);
    String statusDate = statusQuery.getString(ENTITY_STATUS_DATE);
    int flightNumber = statusQuery.getInt(ENTITY_FLIGHT_NUMBER);

    FlightStatusQuery flightStatusQuery = new FlightStatusQuery.Builder()
            .setFlightNumber(flightNumber)
            .setSource(srcAirport)
            .setDestination(destAirport)
            .setStatusDate(statusDate)
            .build();

    SlangFlightAssistant.notifyNonVoiceStatusQuery(flightStatusQuery);

  }

  @ReactMethod
  public void setUserProperties(String userId, ReadableMap rnUserProperties) {
      try {
          Map<String, String> userProperties = new HashMap<>();
          if (null != rnUserProperties && rnUserProperties.toHashMap().size() > 0) {
              for (ReadableMapKeySetIterator it = rnUserProperties.keySetIterator(); it.hasNextKey();) {
                  String key = it.nextKey();
                  userProperties.put(key, rnUserProperties.getString(key));
              }
          }
          SlangFlightAssistant.setUserProperties(userId, userProperties);
      } catch (Exception ex) {
          Log.e("SlangFlightAssistant", "Failed to record user properties:" + ex.getMessage());
      }
  }


  private String getStringConfig(String key) {
    try {
      return this.mConfigOptions != null && this.mConfigOptions.hasKey(key)
          ? this.mConfigOptions.getString(key)
          : null;
    } catch (Exception e) {
      Log.d(TAG, "mConfigOptions encountered an error");
    }

    return null;
  }

  private List<String> getStringListConfig(String key) {
    if (this.mConfigOptions == null && !this.mConfigOptions.hasKey(key))
      return null;

    try {
      ReadableArray stringArray = this.mConfigOptions.getArray(key);
      if (null == stringArray || stringArray.size() == 0)
        return null;

      List<String> stringList = new ArrayList<>(stringArray.size());
      for (int i = 0; i < stringArray.size(); i++) {
        stringList.add(stringArray.getString(i));
      }
      return stringList;
    } catch (Exception e) {
      Log.d(TAG, "mConfigOptions encountered an error");
    }

    return null;
  }

  private WritableMap mapFlightStatusQuery(FlightStatusQuery flightStatusQuery) {
    WritableMap statusQuery = Arguments.createMap();
    if (flightStatusQuery != null) {
      statusQuery.putString(ENTITY_SRC_AIRPORT, flightStatusQuery.getSource().getCode());
      statusQuery.putString(ENTITY_DEST_AIRPORT, flightStatusQuery.getDestination().getCode());
      statusQuery.putString(ENTITY_STATUS_DATE, new SimpleDateFormat("yyyy-MM-dd").format(flightStatusQuery.getDate()));
      statusQuery.putInt(ENTITY_FLIGHT_NUMBER, flightStatusQuery.getFlightNumber());
    }

    return statusQuery;
  }

  private void sendEvent(String name, WritableMap params) {
    this.mReactContext
        .getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
        .emit(name, params);
  }
}
